package fmon.builder

import java.io._

import javafx.application.Application
import javafx.fxml.FXML
import javafx.fxml.FXMLLoader
import javafx.fxml.JavaFXBuilderFactory
import javafx.scene.{control => jfxsc, layout => jfxsl, Parent, Scene}
import javafx.stage.Stage

import scalafx.Includes._
import scalafx.beans.property._
import scalafx.collections.ObservableBuffer
import scalafx.scene.control._
import scalafx.scene.image.ImageView
import scalafx.scene.input.MouseEvent
import scalafx.scene.paint.Color
import scalafx.scene.layout._

import fmon.draw.tile._
import fmon.util.Direction

import Direction._

import scala.math.{min, max}

class MapBuilder {
  @FXML var tileSelector: jfxsl.TilePane = _
  @FXML var gameMap: jfxsl.TilePane = _
  
  var tileSeq: IndexedSeq[AutoFloorTile] = _
  var imgSeq: IndexedSeq[ImageView] = _
  
  var currTile: AutoFloorTile = _
  
  val numRows = 10
  val numCols = 10
  
  @FXML
  def initialize(): Unit = {
    val file = new File(raw"C:\Users\dalyj\Documents\Design\Images\AutoTiles\tilea2.png")
    val palette = new AutoTilePalette(file, 32)
    val tiles = for (y <- 0 until 4; x <- 0 until 8) yield {
      palette(x, y)
    }
    val icons = tiles.zipWithIndex.map{case(t, i) => {
      val view = new ImageView(t.icon.croppedImage())
      view.handleEvent(MouseEvent.Any) {
        e: MouseEvent => if (e.eventType == MouseEvent.MouseClicked) {
          currTile = t
        }
      }
      view.delegate
    }}
    tileSelector.children ++= icons
    currTile = tiles.head
    
    tileSeq = for (y <- 0 until numRows; x <- 0 until numCols) yield currTile
    imgSeq = for (y <- 0 until numRows; x <- 0 until numCols) yield {
      val view = new ImageView(currTile.icon.croppedImage()) // TODO Tile with adjacent
      view.handleEvent(MouseEvent.Any) {
        e: MouseEvent => if (e.eventType == MouseEvent.MouseClicked) {
          val index = point2index(x, y)
          tileSeq = tileSeq.updated(index, currTile)
          autotile(x, y)
        }
      }
      view
    }
    gameMap.prefColumns = numCols
    gameMap.prefRows = numRows
    gameMap.children ++= imgSeq.map(_.delegate)
    
    for (y <- 0 until numRows; x <- 0 until numCols) {
      smoothTile(x, y)
    }
  }
  
  def autotile(x: Int, y: Int) = {
    for (yy <- max(y - 1, 0) to min(y + 1, numCols - 1); xx <- max(x - 1, 0) to min(x + 1, numRows - 1)) {
      smoothTile(xx, yy)
    }
  }
  
  def smoothTile(x: Int, y: Int) = {
    val i = point2index(x, y)
    val dirs = scala.collection.mutable.Set[Direction.Value]()
    
    val onLeft = x == 0
    val onRight = x + 1 == numCols
    val onTop = y == 0
    val onBottom = y + 1 == numRows
    
    if (onLeft || tileSeq(left(i)) == tileSeq(i)) dirs += West
    if (onRight || tileSeq(right(i)) == tileSeq(i)) dirs += East 
    if (onTop || tileSeq(up(i)) == tileSeq(i)) dirs += North
    if (onBottom || tileSeq(down(i)) == tileSeq(i)) dirs += South
    if (onLeft || onTop || tileSeq(left(up(i))) == tileSeq(i)) dirs += Northwest
    if (onLeft || onBottom || tileSeq(down(left(i))) == tileSeq(i)) dirs += Southwest
    if (onTop || onRight || tileSeq(right(up(i))) == tileSeq(i)) dirs += Northeast
    if (onBottom || onRight || tileSeq(right(down(i))) == tileSeq(i)) dirs += Southeast
    val index = point2index(x, y) 
    imgSeq(index).image = tileSeq(index).build(dirs.toSet).croppedImage()
  }
  
  def point2index(x: Int, y: Int) = y * numCols + x
  
  def left(i: Int) = i - 1
  def right(i: Int) = i + 1
  def up(i: Int) = i - numCols
  def down(i: Int) = i + numCols
}

object MapBuilder {
  class MBA extends Application {
    override def start(primaryStage: Stage): Unit = {
      val frameLoader = new FXMLLoader(getClass.getResource("MapBuilder.fxml"))
      val root: Parent = frameLoader.load()
      val controller = frameLoader.getController[MapBuilder]()
      
      //val builderLoader = new FXMLLoader(getClass.getResource("ElementBuilder.fxml"))
      //val builder: Parent = builderLoader.load()
      //val builderController = builderLoader.getController[ElementBuilder]()
      
      //controller.pane.children = builder
      //controller.builder = builderController
      
      val scene: Scene = new Scene(root)
      primaryStage.setScene(scene)
      primaryStage.show()
    }
  }
  
  def main(args: Array[String]): Unit = {
    Application.launch(classOf[MBA], args: _*)
  }
}